unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DirectShow, Menus, VCap, StdCtrls, ExtCtrls, Placemnt, ComCtrls, Buttons,
  Mask, ToolEdit;

type
  TFormMain = class(TForm)
    StatusBar: TStatusBar;
    Panel2: TPanel;
    Panel3: TPanel;
    cap: TVideoCapture;
    Image: TImage;
    FormStorage: TFormStorage;
    Panel6: TPanel;
    btStart: TBitBtn;
    btStop: TBitBtn;
    btCaptureFrame: TBitBtn;
    Panel4: TPanel;
    Label7: TLabel;
    lbDialogs: TListBox;
    Panel5: TPanel;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    Label5: TLabel;
    cbV: TComboBox;
    cbVModes: TComboBox;
    cbA: TComboBox;
    cbAModes: TComboBox;
    feAVI: TFilenameEdit;
    feBitmap: TFilenameEdit;
    cbVCompressors: TComboBox;
    cbACompressors: TComboBox;
    Timer: TTimer;
    cbCaptureAudio: TCheckBox;
    cbVCompr: TCheckBox;
    cbACompr: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure cbVDropDown(Sender: TObject);
    procedure cbADropDown(Sender: TObject);
    procedure cbChange(Sender: TObject);
    procedure capCaptureProgress(Sender: TObject);
    procedure btStartClick(Sender: TObject);
    procedure btStopClick(Sender: TObject);
    procedure btCaptureFrameClick(Sender: TObject);
    procedure capBitmapGrabbed(CapturedImage: TBitmap);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure capChangeDevice(Sender: TObject);
    procedure cbVModesChange(Sender: TObject);
    procedure cbAModesChange(Sender: TObject);
    procedure lbDialogsDblClick(Sender: TObject);
    procedure capStartPreview(Sender: TObject);
    procedure cbVCompressorsDropDown(Sender: TObject);
    procedure cbACompressorsDropDown(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure ChangeDevices(Sender: TObject);
    procedure ChangeCompressors(Sender: TObject);
  private
    function InitCapture : Boolean;
  public
  end;

var
  FormMain: TFormMain;

implementation

uses VCLUtils;

{$R *.DFM}

function TFormMain.InitCapture: Boolean;
var
  s: TStringList;
begin
  Result:= cap.Init;
  if not Result then Exit;

  s:= GetVideoDevicesList;
  try
    cbV.Items.Assign(s);
  finally
    s.Free;
  end;

  s:= GetAudioDevicesList;
  try
    cbA.Items.Assign(s);
  finally
    s.Free;
  end;

  s:= GetVideoCompressorsList;
  try
    cbVCompressors.Items.Assign(s);
  finally
    s.Free;
  end;

  s:= GetAudioCompressorsList;
  try
    cbACompressors.Items.Assign(s);
  finally
    s.Free;
  end;

  Result:= true;
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  if not InitCapture then begin
    ShowMessage('Can''t init capture!');
    Application.Terminate;
  end;
end;

procedure TFormMain.cbVDropDown(Sender: TObject);
var
  s: TStringList;
begin
  s:= GetVideoDevicesList(true);
  try
    cbV.Items.Assign(s);
  finally
    s.Free;
  end;
end;

procedure TFormMain.cbADropDown(Sender: TObject);
var
  s: TStringList;
begin
  s:= GetAudioDevicesList(true);
  try
    cbA.Items.Assign(s);
  finally
    s.Free;
  end;
end;

procedure TFormMain.cbVCompressorsDropDown(Sender: TObject);
var
  s: TStringList;
begin
  s:= GetVideoCompressorsList(true);
  try
    cbVCompressors.Items.Assign(s);
  finally
    s.Free;
  end;
end;

procedure TFormMain.cbACompressorsDropDown(Sender: TObject);
var
  s: TStringList;
begin
  s:= GetAudioCompressorsList(true);
  try
    cbACompressors.Items.Assign(s);
  finally
    s.Free;
  end;
end;

procedure TFormMain.cbChange(Sender: TObject);
begin
  ChangeDevices(Self);
end;

procedure TFormMain.capCaptureProgress(Sender: TObject);
begin
  StatusBar.SimpleText:= 'Frames Captured '+IntToStr(cap.NotDropped)+
                         '. Frames Dropped '+IntToStr(cap.FramesDropped)+'.';
  btStart.Enabled:= not cap.Capturing;
  btStop.Enabled:= not btStart.Enabled;
end;

procedure TFormMain.btStartClick(Sender: TObject);
begin
  cap.CaptureFileName:= feAVI.FileName;
  cap.StartCapture(true);
  capCaptureProgress(Self);
end;

procedure TFormMain.btStopClick(Sender: TObject);
begin
  cap.StopCapture;
  capCaptureProgress(Self);
end;

procedure TFormMain.btCaptureFrameClick(Sender: TObject);
begin
  if not cap.CaptureFrame then begin
    ShowMessage('Can''t capture frame!');
  end;
end;

procedure TFormMain.capBitmapGrabbed(CapturedImage: TBitmap);
begin
  CapturedImage.SaveToFile(feBitmap.FileName);
  Image.Picture.Assign(CapturedImage);
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  cap.StopPreview;
  cap.StopCapture;
end;

procedure TFormMain.capChangeDevice(Sender: TObject);
var
  i, capCount: integer;
  cvcapMode, vcapMode: TVCapMode;
  cacapMode, acapMode: TACapMode;
  s: string;
  d: TCaptureDialog;
begin
  cvcapMode:= cap.VCapMode;
  cacapMode:= cap.ACapMode;

  cbVModes.Items.Clear;
  capCount:= cap.VCapModeCount;
  for i:= 0 to capCount-1 do begin
    vcapMode:= cap.VCapModes[i];
    s:= GetModeString(vcapMode);
    cbVModes.Items.Add(s);
    if IsEqualModes(cvcapMode, vcapMode) then cbVModes.ItemIndex:= i;
  end;

  cbAModes.Items.Clear;
  capCount:= cap.ACapModeCount;
  for i:= 0 to capCount-1 do begin
    acapMode:= cap.ACapModes[i];
    s:= GetModeString(acapMode);
    cbAModes.Items.Add(s);
    if IsEqualModes(cacapMode, acapMode) then cbAModes.ItemIndex:= i;
  end;

  lbDialogs.Items.Clear;
  for d:= Low(TCaptureDialog) to High(TCaptureDialog) do
    if d in cap.Dialogs then begin
      lbDialogs.Items.AddObject(DialogTitles[d], TObject(d));
    end;

end;

procedure TFormMain.capStartPreview(Sender: TObject);
// refresh cur modes
var
  i, capCount: integer;
  cvcapMode, vcapMode: TVCapMode;
  cacapMode, acapMode: TACapMode;
begin
  cvcapMode:= cap.VCapMode;
  cacapMode:= cap.ACapMode;

  capCount:= cap.VCapModeCount;
  for i:= 0 to capCount-1 do begin
    vcapMode:= cap.VCapModes[i];
    if IsEqualModes(cvcapMode, vcapMode) then cbVModes.ItemIndex:= i;
  end;

  capCount:= cap.ACapModeCount;
  for i:= 0 to capCount-1 do begin
    acapMode:= cap.ACapModes[i];
    if IsEqualModes(cacapMode, acapMode) then cbAModes.ItemIndex:= i;
  end;
end;

procedure TFormMain.cbVModesChange(Sender: TObject);
begin
  cap.SetVCapMode(cbVModes.ItemIndex);
end;

procedure TFormMain.cbAModesChange(Sender: TObject);
begin
  cap.SetACapMode(cbAModes.ItemIndex);
end;

procedure TFormMain.lbDialogsDblClick(Sender: TObject);
begin
  if (lbDialogs.Items.Count>0) and (lbDialogs.ItemIndex>=0) then
    cap.ShowDialog(TCaptureDialog(lbDialogs.Items.Objects[lbDialogs.ItemIndex]));
end;

procedure TFormMain.TimerTimer(Sender: TObject);
begin
  // make a delay to show form (let form get a handle)
  Timer.Enabled:= false;
  if (cbV.Items.Count>0) then cbV.ItemIndex:= 0;
  if (cbA.Items.Count>0) then cbA.ItemIndex:= 0;
  ChangeDevices(Self);
end;

procedure TFormMain.ChangeDevices(Sender: TObject);
begin
  cap.WantAudio:= cbCaptureAudio.Checked;
  cbVCompr.Checked:= false;
  cbACompr.Checked:= false;
  StartWait;
  try
    cap.ChooseDevices(cbV.ItemIndex, cbA.ItemIndex);
  finally
    StopWait;
  end;
end;

procedure TFormMain.ChangeCompressors(Sender: TObject);
var
  vcomp, acomp: integer;
begin
  vcomp:= -1; acomp:= -1;
  if cbVCompr.Checked then vcomp:= cbVCompressors.ItemIndex;
  if cbACompr.Checked then acomp:= cbACompressors.ItemIndex;
  StartWait;
  try
    cap.ChooseCompressors(vcomp, acomp);
  finally
    StopWait;
  end;
end;

end.
